@@ -9,6 +9,7 @@ import re
 import sys
 import textwrap
 import token
+from typing import List
 import unittest
 from time import time
 
@@ -18,6 +19,11 @@ from asttokens import util, ASTTokens
 
 from . import tools
 
+try:
+  from astroid.nodes.utils import Position as AstroidPosition
+except Exception:
+  AstroidPosition = ()
+
 
 class TestMarkTokens(unittest.TestCase):
   maxDiff = None
@@ -30,7 +36,7 @@ class TestMarkTokens(unittest.TestCase):
 
   def create_mark_checker(self, source, verify=True):
     atok = self.create_asttokens(source)
-    checker = tools.MarkChecker(atok)
+    checker = tools.MarkChecker(atok, self.is_astroid_test)
 
     # The last token should always be an ENDMARKER
     # None of the nodes should contain that token
@@ -133,7 +139,10 @@ b +     # line3
     m = self.create_mark_checker(source, verify=False)
     tested_nodes = m.verify_all_nodes(self)
 
-    exp_index = (0 if six.PY2 else 1) + (2 if self.is_astroid_test else 0)
+    exp_index = (0 if six.PY2 else 1) + (3 if self.is_astroid_test else 0)
+    # For ast on Python 3.9, slices are expressions, we handle them and test them.
+    if not self.is_astroid_test and issubclass(ast.Slice, ast.expr):
+      exp_index += 1
     exp_tested_nodes = self.expect_tested_nodes[path][exp_index]
     self.assertEqual(tested_nodes, exp_tested_nodes)
 
@@ -141,21 +150,21 @@ b +     # line3
   # There is not too much need to verify these counts. The main reason is: if we find that some
   # change reduces the count by a lot, it's a red flag that the test is now covering fewer nodes.
   expect_tested_nodes = {
-    #                                   AST       | Astroid
-    #                                   Py2   Py3 | Py2   Py3
-    'astroid/__init__.py':            ( 4,    4,    4,    4,   ),
-    'astroid/absimport.py':           ( 4,    3,    4,    3,   ),
-    'astroid/all.py':                 ( 21,   23,   21,   23,  ),
-    'astroid/clientmodule_test.py':   ( 75,   67,   69,   69,  ),
-    'astroid/descriptor_crash.py':    ( 30,   28,   30,   30,  ),
-    'astroid/email.py':               ( 3,    3,    1,    1,   ),
-    'astroid/format.py':              ( 64,   61,   62,   62,  ),
-    'astroid/module.py':              ( 185,  174,  171,  171, ),
-    'astroid/module2.py':             ( 248,  253,  240,  253, ),
-    'astroid/noendingnewline.py':     ( 57,   59,   57,   63,  ),
-    'astroid/notall.py':              ( 15,   17,   15,   17,  ),
-    'astroid/recursion.py':           ( 6,    6,    4,    4,   ),
-    'astroid/suppliermodule_test.py': ( 20,   17,   18,   18,  ),
+    #                                   AST                  | Astroid
+    #                                   Py2   Py3  Py3+slice | Py2   Py3
+    'astroid/__init__.py':            ( 4,    4,   4,          4,    4,   ),
+    'astroid/absimport.py':           ( 4,    3,   3,          4,    3,   ),
+    'astroid/all.py':                 ( 21,   23,  23,         21,   23,  ),
+    'astroid/clientmodule_test.py':   ( 75,   67,  67,         69,   69,  ),
+    'astroid/descriptor_crash.py':    ( 30,   28,  28,         30,   30,  ),
+    'astroid/email.py':               ( 3,    3,   3,          1,    1,   ),
+    'astroid/format.py':              ( 64,   61,  61,         62,   62,  ),
+    'astroid/module.py':              ( 185,  174, 174,        171,  171, ),
+    'astroid/module2.py':             ( 248,  253, 255,        240,  253, ),
+    'astroid/noendingnewline.py':     ( 57,   59,  59,         57,   63,  ),
+    'astroid/notall.py':              ( 15,   17,  17,         15,   17,  ),
+    'astroid/recursion.py':           ( 6,    6,   6,          4,    4,   ),
+    'astroid/suppliermodule_test.py': ( 20,   17,  17,         18,   18,  ),
   }
 
   # This set of methods runs verifications for the variety of syntax constructs used in the
@@ -226,7 +235,7 @@ b +     # line3
 
   def test_slices(self):
     # Make sure we don't fail on parsing slices of the form `foo[4:]`.
-    source = "(foo.Area_Code, str(foo.Phone)[:3], str(foo.Phone)[3:], foo[:], bar[::2, :], [a[:]][::-1])"
+    source = "(foo.Area_Code, str(foo.Phone)[:3], str(foo.Phone)[3:], foo[:], bar[::2, :], bar2[:, ::2], [a[:]][::-1])"
     m = self.create_mark_checker(source)
     self.assertIn("Tuple:" + source, m.view_nodes_at(1, 0))
     self.assertEqual(m.view_nodes_at(1, 1),
@@ -239,7 +248,7 @@ b +     # line3
     # important, so we skip them here.
     self.assertEqual({n for n in m.view_nodes_at(1, 56) if 'Slice:' not in n},
                      { "Subscript:foo[:]", "Name:foo" })
-    self.assertEqual({n for n in m.view_nodes_at(1, 64) if 'Slice:' not in n},
+    self.assertEqual({n for n in m.view_nodes_at(1, 64) if 'Slice:' not in n and 'Tuple:' not in n},
                      { "Subscript:bar[::2, :]", "Name:bar" })
 
   def test_adjacent_strings(self):
@@ -281,13 +290,14 @@ bar = ('x y z'   # comment2
       # Test of https://bitbucket.org/plas/thonny/issues/162/weird-range-marker-crash-with-non-ascii
       # Only on PY3 because Py2 doesn't support unicode identifiers.
       for source in (
+        "℘·2=1+a℘·b+a℘·2b",  # example from https://github.com/python/cpython/issues/68382
         "sünnikuupäev=str((18+int(isikukood[0:1])-1)//2)+isikukood[1:3]",
         "sünnikuupaev=str((18+int(isikukood[0:1])-1)//2)+isikukood[1:3]"):
         m = self.create_mark_checker(source)
         self.assertEqual(m.view_nodes_at(1, 0), {
           "Module:%s" % source,
           "Assign:%s" % source,
-          "%s:%s" % ("AssignName" if self.is_astroid_test else "Name", source[:12])
+          "%s:%s" % ("AssignName" if self.is_astroid_test else "Name", source.split("=")[0]),
         })
 
 
@@ -437,6 +447,16 @@ bar = ('x y z'   # comment2
     self.assertEqual(m.view_nodes_at(2, 4), {'Name:x', 'Subscript:x[4]'})
 
   if not six.PY2:
+    def test_bad_tokenless_types(self):
+      # Cases where _get_text_positions_tokenless is incorrect in 3.8.
+      source = textwrap.dedent("""
+        def foo(*, name: str):  # keyword-only argument with type annotation
+          pass
+
+        f(*(x))  # ast.Starred with parentheses
+      """)
+      self.create_mark_checker(source)
+
     def test_return_annotation(self):
       # See https://bitbucket.org/plas/thonny/issues/9/range-marker-crashes-on-function-return
       source = textwrap.dedent("""
@@ -549,11 +569,14 @@ a; b; c(
   17
 ); d # comment1; comment2
 if 2: a; b; # comment3
+if a:
+  if b: c; d  # comment4
     """
     m = self.create_mark_checker(source)
     self.assertEqual(
       [m.atok.get_text(n) for n in m.all_nodes if util.is_stmt(n)],
-      ['a', 'b', 'c(\n  17\n)', 'd', 'if 2: a; b', 'a', 'b'])
+      ['a', 'b', 'c(\n  17\n)', 'd', 'if 2: a; b', 'a', 'b',
+       'if a:\n  if b: c; d', 'if b: c; d', 'c', 'd'])
 
 
   def test_complex_numbers(self):
@@ -631,8 +654,19 @@ j  # not a complex number, just a name
         except OSError:
           continue
 
-        # Astroid fails with a syntax error if a type comment is on its own line
-        if self.is_astroid_test and re.search(r'^\s*# type: ', source, re.MULTILINE):
+        if self.is_astroid_test and (
+            # Astroid fails with a syntax error if a type comment is on its own line
+            re.search(r'^\s*# type: ', source, re.MULTILINE)
+            # Astroid can fail on this file, specifically raising an exception at this line of code:
+            #     lambda node: node.name == "NamedTuple" and node.parent.name == "typing"
+            # with the error:
+            #     AttributeError: 'If' object has no attribute 'name'
+            # See https://github.com/gristlabs/asttokens/runs/7602147792
+            # I think the code that causes the problem is:
+            #     if sys.version_info >= (3, 11):
+            #         NamedTuple = typing.NamedTuple
+            or filename.endswith("typing_extensions.py")
+        ):
           print('Skipping', filename)
           continue
 
@@ -716,6 +750,32 @@ if all((nonblank := line).strip() == '' for line in lines):
 partial_sums = [total := total + v for v in values]
 """)
 
+  if sys.version_info >= (3, 10):
+    def test_match_case(self):
+      m = self.create_mark_checker("""
+if 0:
+  match x:
+    case ast.BinOp():
+      if z:
+        pass
+    case cls(a,b) if y:
+      pass
+    case _:
+      match y:
+        case 1:
+          pass
+        case _:
+          pass
+""")
+      self.assertEqual(m.view_nodes_at(10, 6), {
+        'Match:'
+        '      match y:\n'
+        '        case 1:\n'
+        '          pass\n'
+        '        case _:\n'
+        '          pass',
+      })
+
   def parse_snippet(self, text, node):
     """
     Returns the parsed AST tree for the given text, handling issues with indentation and newlines
@@ -729,11 +789,14 @@ partial_sums = [total := total + v for v in values]
     # string contained in an astroid.Const or astroid.Expr it will end up in the doc attribute and be
     # a pain to extract for comparison
     # For starred expressions, e.g. `*args`, we wrap it in a function call to make it parsable.
+    # For slices, e.g. `x:`, we wrap it in an indexing expression to make it parsable.
     indented = re.match(r'^[ \t]+\S', text)
     if indented:
       return self.module.parse('def dummy():\n' + text).body[0].body[0]
     if util.is_starred(node):
       return self.module.parse('f(' + text + ')').body[0].value.args[0]
+    if util.is_slice(node):
+      return self.module.parse('a[' + text + ']').body[0].value.slice
     if util.is_expr(node):
       return self.module.parse('_\n(' + text + ')').body[1].value
     if util.is_module(node):
@@ -769,7 +832,7 @@ partial_sums = [total := total + v for v in values]
           'def foo():\n    """xx"""\n    None')
 
   nodes_classes = ast.AST
-  context_classes = [ast.expr_context]
+  context_classes = [ast.expr_context] # type: List[util.AstNode]
   iter_fields = staticmethod(ast.iter_fields)
 
   def assert_nodes_equal(self, t1, t2):
@@ -782,6 +845,10 @@ partial_sums = [total := total + v for v in values]
     else:
       self.assertEqual(type(t1), type(t2))
 
+    if isinstance(t1, AstroidPosition):
+      # Ignore the lineno/col_offset etc. from astroid
+      return
+
     if isinstance(t1, (list, tuple)):
       self.assertEqual(len(t1), len(t2))
       for vc1, vc2 in zip(t1, t2):
